# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.staticfiles import finders
from django.urls import reverse
from django.conf import settings

import os

import qrcode
from PIL import Image, ImageDraw, ImageFont
from io import BytesIO


class Rectangle:
    def __init__(self, left, top, width, height):

        self.left = left
        self.top = top
        self.width = width
        self.height = height

    def to_rect(self):
        return [(self.left, self.top), (self.left + self.width, self.top + self.height)]


# Utilisé lors de la création de billets pour en valider la taille (cf. guichet/forms.py)
MIN_TEMPLATE_SIZE = Rectangle(0, 0, 2400, 1100)


frames = {
    "payee": Rectangle(45, 854, 440, 60),
    "note": Rectangle(45, 934, 440, 60),
    "ticket": Rectangle(45, 1014, 440, 60),
}

font_sizes = {"payee": 60, "note": 60, "ticket": 60}

# Utilise une police monospace
font_name = "Inconsolata-Bold.ttf"
font_path = finders.find(os.path.join("fonts", font_name))

fonts = {}


def font_size(rect, text, size, d):
    if size not in fonts.keys():
        font = ImageFont.truetype(font_path, size)
        fonts[size] = d.textsize("0", font), font

    char_width, char_height = fonts[size][0]
    return (
        size
        if char_width * len(text) < rect.width and char_height < rect.height
        else font_size(rect, text, size - 1, d)
    )


def write_text(rect, text, font, d):
    d.rectangle(rect.to_rect(), fill="white")
    d.text(
        (rect.left + rect.width / 2 - len(text) * font[0][0] / 2, rect.top + rect.height / 2 - font[0][1] / 2),
        text,
        font=font[1],
        fill=(0, 0, 0, 255),
    )


def print_sandbox_billet(ticket, format="JPEG"):
    return create_billet(
        ("%" + str(ticket.id)).ljust(6, "%"), ticket.name, "Billet de test", "(non valide)", ticket.design.path, format
    )


def registration_to_image(registration, format="JPEG"):

    ticket = registration.ticket.name
    if registration.payee:
        payee = registration.payee
        note = "({})".format(registration.owner.get_full_name())
    else:
        payee = registration.owner.get_full_name()
        if registration.owner.login_inp_net:
            note = "({})".format(registration.owner.login_inp_net)
        else:
            note = "(extérieur)"

    template_path = registration.ticket.design.path

    # Build a qr code that link to the validation page of the registration
    # (this is "safe" as only a manager is able to validate a ticket using this url)
    qr_code = settings.BASE_URL + reverse(
        "check-registration-with-uuid",
        kwargs={"event_id": registration.ticket.event.id, "registration_uuid": registration.qrcode_uuid},
    )

    return create_billet(qr_code, ticket, payee, note, template_path, format)


def create_billet(qr_code, ticket, payee, note, template_path, format):

    template = Image.open(template_path).convert("RGBA")
    # On fait une image vide pour le texte
    txt = Image.new("RGBA", template.size, (255, 255, 255, 0))

    # Le context pour écrire
    d = ImageDraw.Draw(txt)

    font_sizes["payee"] = font_size(frames["payee"], payee, font_sizes["payee"], d)
    write_text(frames["payee"], payee, fonts[font_sizes["payee"]], d)

    font_sizes["note"] = font_size(frames["note"], note, font_sizes["note"], d)
    write_text(frames["note"], note, fonts[font_sizes["note"]], d)

    font_sizes["ticket"] = font_size(frames["ticket"], ticket, font_sizes["ticket"], d)
    write_text(frames["ticket"], ticket, fonts[font_sizes["ticket"]], d)

    qr = qrcode.QRCode(version=10, error_correction=qrcode.constants.ERROR_CORRECT_H, box_size=10, border=4)
    qr.add_data(qr_code)
    qr.make(fit=True)

    png_qr_code = qr.make_image(fill_color="black", back_color="white")

    billet_complete = Image.alpha_composite(template, txt)
    tmp = BytesIO()
    billet_complete.save(tmp, "PNG")
    tmp.seek(0)
    tmp_load = Image.open(tmp)
    tmp_load.paste(png_qr_code, (45, 50))
    buff = BytesIO()
    tmp_load.convert("RGB").save(buff, format=format)
    billet_complete_png = buff.getvalue()
    buff.close()

    return billet_complete_png
